from ngsolve import *
import numpy as np
import netgen.gui

writeDat = False
pi = np.pi

mesh = Mesh("team7.vol")

mesh.ngmesh.SetMaterial (1, "Coil")
mesh.ngmesh.SetMaterial (2, "Plate") 
mesh.ngmesh.SetMaterial (3, "Air")

# physics constants
mu0 = 4.*pi*1.0e-7
f = 200.  # Hz
mus = {'Coil': 1.*mu0, 'Plate': 1.*mu0, 'Air': 1.*mu0}
sigmas = {'Coil': 1e-5, 'Plate': 3.526e7, 'Air': 1e-5}
curdens = 2742 / (0.1 * 0.025) # Amper turns / coil area

# Finite element space
V = HCurl(mesh, order=1, nograds = True, complex = True)
VB = HCurl(mesh, order = 1)


# magnetic permeability
muValues = {mat: mus[mat] for mat in mesh.GetMaterials()}
mu = CoefficientFunction([muValues[mat]
                          for mat in mesh.GetMaterials()])  

# conductivity
omega_sigmaValues = {mat: sigmas[mat]*2.*pi*f for mat in mesh.GetMaterials()}
omega_sigma = CoefficientFunction([omega_sigmaValues[mat]
                             for mat in mesh.GetMaterials()]) 

# Trial function
u = V.TrialFunction()
# Test function
v = V.TestFunction()

A_plate = GridFunction(V)
B = GridFunction(VB)
J = GridFunction(VB)
uh = GridFunction(V)

a = BilinearForm(V)
a += SymbolicBFI(1 / mu * InnerProduct(curl(u),curl(v)))
a += SymbolicBFI(1j * omega_sigma * InnerProduct(u, v))


source = CoefficientFunction((-y,x,0))
source_norm = 1 / Norm(source) * source
T0 = mesh.MaterialCF({"Coil": source_norm * curdens}, default = (0,0,0))

f = LinearForm(V)
f += SymbolicLFI(InnerProduct(curl(u), T0), definedon = mesh.Materials('Coil'))

with TaskManager():
    a.Assemble()
    f.Assemble()
    uh.vec.data = a.mat.Inverse() * f.vec

A_plate.Set(uh, definedon = mesh.Materials('Plate'))


Draw(curl(uh), mesh, "B")
Draw(A_plate, mesh, "A_plate")
Draw(omega_sigma * A_plate.real, mesh, "J")

if writeDat:
    B.Set(curl(uh).real)
    with open("NGSolve_B1_Team7_own.dat", 'w') as fd:
        fd.write("#x y z Bx By Bz\n")
        for i_x in np.linspace(0.,288e-3,num=100):
            B_out = B(mesh(i_x, 0.072, 0.034))
            fd.write("%.15e %.15e %.15e %.15e %.15e %.15e\n" % (i_x,0.072,0.034,B_out[0],B_out[1],B_out[2]))
        fd.write("\n")